var Barcode = function()
{
	var barcode = {};
	var debug = 0;
	
	var barcodesByName = [];

	// Page initialization
	barcode.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', barcode.validateStep1);
		Events.addEventHandler('bioplate', 'change', barcode.bioPlateOnChange);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', barcode.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', barcode.validateStep2);
		Events.addEventHandler('pool_schema', 'change', barcode.poolSchemaOnChange);
		Events.addEventHandler('barcode_variant', 'change', barcode.barcodeVariantOnChange);
		var wells = document.getElementsByClassName('well');
		for (var wellNo = 0; wellNo < wells.length; wellNo++)
		{
			Events.addEventHandler(wells[wellNo], 'click', barcode.selectBarcode);
		}
		Events.addEventHandler('select-barcode', 'click', barcode.barcodeSelected);
		Events.addEventHandler('wizard', 'click', barcode.hideBarcodeSelection);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', barcode.submit);

		var url = '../LibPrep.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCDNAPlatesForBarcoding';		
		Wizard.showLoadingAnimation('Loading cDNA plates...');
		Wizard.asyncJsonRequest(url, barcode.initializeStep1);
		
		var url = '../LibPrep.servlet?ID='+App.getSessionId();
		url += '&cmd=GetAllBarcodeInfo';
		Wizard.asyncJsonRequest(url, barcode.barcodesLoaded);
	}


	barcode.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];

		var bioplates = response.bioplates;
		var plates = frm.bioplate;
		if (bioplates.length > 0)
		{
			for (var i=0; i < bioplates.length; i++)
			{
				var bioplate = bioplates[i];
				var option = new Option(bioplate.name, bioplate.id);
				option.bioplate = bioplate;
				plates.options[plates.length] = option;
			}
		}
		else
		{
			Wizard.setFatalError('No cDNA bioplates available for processing.');
			return;
		}
		
		bioplateIsValid = true;
		Wizard.setInputStatus('bioplate', 'valid');
		Events.sendChangeEvent('bioplate');

		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	barcode.bioPlateOnChange = function()
	{
		var frm = document.forms['reggie'];
		var bioplate = frm.bioplate[frm.bioplate.selectedIndex].bioplate;
		Doc.element('cdnaComments').innerHTML = Strings.encodeTags(bioplate.comments);
	}

	barcode.barcodesLoaded = function(response)
	{
		var barcodes = response.barcodes
		var selectBarcodeHtml = '';
		
		for (var i = 0; i < barcodes.length; i++)
		{
			if (i % 8 == 0 && i > 0)
			{
				selectBarcodeHtml += '<div class="menuseparator"></div>';
			}
			var barcode = barcodes[i];
			barcodesByName[barcode.name] = barcode;
			selectBarcodeHtml += '<div class="menuitem enabled" id="bc-'+barcode.id+'" data-barcode-name="'+Strings.encodeTags(barcode.name)+'">';
			selectBarcodeHtml += Strings.encodeTags(barcode.name)+'</div>';
		}
		Doc.element('select-barcode-all').innerHTML = selectBarcodeHtml;
	}

	barcode.validateStep1 = function(event)
	{}

	barcode.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];

		var url = '../LibPrep.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCDnaInfoForPlate&bioplate='+frm.bioplate.value;	
		Wizard.showLoadingAnimation('Loading cDNA bioplate information...');
		Wizard.asyncJsonRequest(url, barcode.cdnaInfoLoaded);
	}
	
	barcode.cdnaInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var cdnaPlate = frm.bioplate[frm.bioplate.selectedIndex].bioplate;

		var schema = PoolSchema.getById(cdnaPlate.poolSchema);
		PoolSchema.initList(frm.pool_schema, schema.id);
		PoolSchema.initVariantList(frm.barcode_variant, schema, cdnaPlate.barcodeVariant);
		
		var list = response.cdna;
		// Pre-process the return cDNA items
		for (var i = 0; i < list.length; i++)
		{
			barcode.checkAndPreProcessCDna(list[i]);
		}
		
		barcode.viewAsPlate(list, schema);
		Wizard.keepSessionAlive();

		Wizard.setCurrentStep(2);
		Doc.show('goregister');
		Doc.show('gocancel');
	}

	
	barcode.checkAndPreProcessCDna = function(cdna)
	{
		// Set the 'stratagene'/'external' flag
		cdna.stratagene = Reggie.isStratagene(cdna.name);
		cdna.external = Reggie.isExternal(cdna.name);
	}

	barcode.viewAsPlate = function(list, schema)
	{
		Plate.init(8, 12, schema, WellPainter);

		for (var i = 0; i < list.length; i++)
		{
			var cdna = list[i];
			var well = cdna.bioWell;
			Plate.getWell(well.row, well.column).setExtract(cdna);
		}
		var frm = document.forms['reggie'];
		
		var barcodeVariant = PoolSchema.getBarcodeVariantByName(schema, frm.barcode_variant.value);
		barcode.assignSchemaDefinedBarcodeVariant(schema, barcodeVariant);
		
		WellPainter.barcodeVariant = barcodeVariant;
		Plate.paint(Plate.getWells());
		PoolSchema.buildPoolTableRow(schema, 12, false);
	}

	barcode.assignSchemaDefinedBarcodeVariant = function(schema, barcodeVariant)
	{
		var wells = Plate.getWells();
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			var cdna = well.extract;
			if (cdna)
			{
				var indexSet = barcodeVariant ? barcodeVariant.indexSets[well.column] : null;
				var barcode = null;
				if (indexSet)
				{
					var barcodeName = indexSet.barcodes[well.row];
					barcode = barcodesByName[barcodeName];
				}
				cdna.barcode = barcode;
				cdna.defaultBarcode = barcode;
				well.duplicate = false;
			}
		}
	}
	
	barcode.poolSchemaOnChange = function()
	{
		var frm = document.forms['reggie'];
		var schema = POOL_SCHEMA[frm.pool_schema.selectedIndex];
		Plate.setPoolSchema(schema);
		PoolSchema.initVariantList(frm.barcode_variant, schema);
		
		var barcodeVariant = PoolSchema.getBarcodeVariantByName(schema, frm.barcode_variant.value);
		barcode.assignSchemaDefinedBarcodeVariant(schema, barcodeVariant);
		
		WellPainter.barcodeVariant = barcodeVariant;
		Plate.paint(Plate.getWells());
		PoolSchema.buildPoolTableRow(schema, 12, false);
	}

	barcode.barcodeVariantOnChange = function()
	{
		var frm = document.forms['reggie'];
		var schema = POOL_SCHEMA[frm.pool_schema.selectedIndex];
		var barcodeVariant = PoolSchema.getBarcodeVariantByName(schema, frm.barcode_variant.value);
		
		barcode.assignSchemaDefinedBarcodeVariant(schema, barcodeVariant);

		WellPainter.barcodeVariant = barcodeVariant;
		Plate.paint(Plate.getWells());
	}

	var currentWell;
	var lastSelectedBarcode;
	barcode.selectBarcode = function(event)
	{
		var row = Data.int(event.currentTarget, 'row');
		var column = Data.int(event.currentTarget, 'col');

		// Locate the barcode selection div so that the current
		// barcode is positioned next to the current mouse position
		// Initially set it to the right of the mouse so that the
		// center is vertically aligned
		
		currentWell = Plate.getWell(row, column);
		var cdna = currentWell.extract;
		var scroll = 0;
		if (cdna)
		{
			// Reset 'current' selection
			var menu = Doc.element('select-barcode');
			var selectAll = Doc.element('select-barcode-all');
			for (var i = 0; i <  selectAll.childNodes.length; i++)
			{
				Doc.removeClass(selectAll.childNodes[i], 'current');
			}
			menu.style.display = 'block';

			var x = event.clientX+1;
			var halfHeight = Math.floor(selectAll.offsetHeight/2)
			var y = event.clientY-halfHeight;
			var scroll = 0;
			var barcodeDiv;
			
			var useBarcode = cdna.barcode || lastSelectedBarcode;
			if (useBarcode)
			{
				var barcodeDiv = Doc.element('bc-'+useBarcode.id);
				if (!cdna.barcode && barcodeDiv.nextSibling) barcodeDiv = barcodeDiv.nextSibling;
				// Try to scroll the current barcode so that it's baseline is at the center of the div
				scroll = barcodeDiv.offsetTop + barcodeDiv.offsetHeight - halfHeight;
				if (scroll < 0) 
				{
					// We get a negative scroll for the first few elements, shift the
					// entire selection div down instead
					y -= scroll;
					scroll = 0;
				}
				else if (scroll > selectAll.scrollHeight - selectAll.offsetHeight)
				{
					// We get a too large scroll value for the last few elements, shift
					// the entire selection div up instead
					y -= scroll - (selectAll.scrollHeight - selectAll.offsetHeight);
					scroll = selectAll.scrollHeight - selectAll.offsetHeight;
				}
				if (cdna.barcode)
				{
					Doc.addClass(barcodeDiv, 'current');
				}
			}
			
			// Default barcode
			if (cdna.defaultBarcode && cdna.defaultBarcode != cdna.barcode)
			{
				var selectDefault = Doc.element('select-barcode-default');
				Data.set(selectDefault, 'barcode-name', cdna.defaultBarcode.name);
				selectDefault.innerHTML = 'Default: ' + Strings.encodeTags(cdna.defaultBarcode.name);
				Doc.show('select-barcode-default');
				Doc.show('select-barcode-default-separator');
			}
			else
			{
				Doc.hide('select-barcode-default');
				Doc.hide('select-barcode-default-separator');
			}
			
			// Position the selection div
			selectAll.scrollTop = scroll;
			menu.style.left = (x)+'px';
			menu.style.top = (y)+'px';
			event.stopPropagation();
		}
	}

	barcode.barcodeSelected = function(event)
	{
		var frm = document.forms['reggie'];
		var schema = POOL_SCHEMA[frm.pool_schema.selectedIndex];
		var target = event.target;
		if (currentWell && currentWell.extract)
		{
			lastSelectedBarcode = barcodesByName[Data.get(target, 'barcode-name')];
			currentWell.extract.barcode = lastSelectedBarcode;
			var poolNum = schema.getPoolNumForColumn(currentWell.column);
			if (poolNum >= 0) 
			{
				barcode.checkDuplicateBarcode(poolNum);
				Plate.paint(schema.getWellsInPool(Plate, poolNum));
			}
			else
			{
				Plate.paint([currentWell]);
			}
		}
	}
	
	barcode.hideBarcodeSelection = function()
	{
		Doc.hide('select-barcode');
	}
	
	barcode.checkDuplicateBarcode = function(poolNum)
	{
		var frm = document.forms['reggie'];
		var schema = POOL_SCHEMA[frm.pool_schema.selectedIndex];

		var wells = schema.getWellsInPool(Plate, poolNum);
		var wellByBarcodeName = [];
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			var cdna = well.extract;
			if (cdna && cdna.barcode)
			{
				if (wellByBarcodeName[cdna.barcode.name])
				{
					well.duplicate = true;
					wellByBarcodeName[cdna.barcode.name].duplicate = true;
				}
				else
				{
					wellByBarcodeName[cdna.barcode.name] = well;
					well.duplicate = false;
				}
			}
		}
	}

	barcode.validateStep2 = function(event)
	{
		var frm = document.forms['reggie'];

		var wells = Plate.getWells();
		var numErrors = 0;
		var numWarnings = 0;
		var numCdna = 0;
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			if (well.hasWarning())
			{
				numWarnings++;
			}
			if (well.hasError())
			{
				numErrors++;
			}
			else if (well.extract && well.extract.id)
			{
				numCdna++;
			}
		}

		if (numErrors > 0)
		{
			event.preventDefault();
			return;
		}
		else if (numCdna == 0)
		{
			alert('There is no cDNA in any wells');
			event.preventDefault();
			return;
		}
		else if (numWarnings > 0)
		{
			if (!frm.verifyGoNext || !frm.verifyGoNext.checked)
			{
				Wizard.showGoNextConfirmation(true, 'There are ' + numWarnings + ' wells with a warning. Continue anyway?');
				event.preventDefault();
			}
		}
	}
	
	barcode.submit = function()
	{
		var frm = document.forms['reggie'];

		var submitInfo = {};
		var plateInfo = {};
		
		submitInfo.bioplate = plateInfo;
		plateInfo.id = parseInt(frm.bioplate.value);
		plateInfo.poolSchema = frm.pool_schema.value;
		plateInfo.barcodeVariant = frm.barcode_variant.value;
		plateInfo.comments = frm.comments.value;
		plateInfo.wells = [];

		var wells = Plate.getWells();
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			if (well.extract && well.extract.id)
			{
				var tmp = {};
				tmp.row = well.row;
				tmp.column = well.column;
				tmp.cdna = {};
				tmp.cdna.id = well.extract.id;
				tmp.cdna.barcode = {};
				tmp.cdna.barcode.id = well.extract.barcode.id;
				plateInfo.wells[plateInfo.wells.length] = tmp;
			}
		}

		var url = '../LibPrep.servlet?ID='+App.getSessionId();
		url += '&cmd=CreateBarcodedLibraries';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, barcode.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	barcode.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return barcode;
}();

Doc.onLoad(Barcode.initPage);

var WellPainter = function()
{
	var painter = {};
	
	painter.getClassNameForWell = function(well, schema)
	{
		var cls = '';
		var indexSet = painter.barcodeVariant ? painter.barcodeVariant.indexSets[well.column] : null;
		if (indexSet)
		{
			cls += indexSet.color;
		}
		var cdna = well.extract;
		if (cdna && cdna.barcode)
		{
			if (cdna.defaultBarcode && cdna.barcode != cdna.defaultBarcode)
			{
				cls += ' bg-modified';
			}
		}
		if (well.duplicate)
		{
			cls += ' duplicate';
		}
		return cls;
	}
	
	painter.getWellText = function(well, schema)
	{
		var text = '';
		var cdna = well.extract;
		if (cdna)
		{
			var name = cdna.name;
			var i = name.indexOf('.m');
			text += '<div class="name">'+name.substring(0, i)+'.<br>&nbsp;'+name.substring(i)+'</div>';
			if (cdna.barcode)
			{
				text += '<div class="barcode">'+cdna.barcode.name+'</div>';
				if (cdna.defaultBarcode && cdna.barcode != cdna.defaultBarcode)
				{
					text += '<div class="warn-msg">Modified barcode</div>';
				}
			}
			else
			{
				well.setError('No barcode');
			}
		}
		if (well.duplicate)
		{
			well.setError('Duplicate barcode');
		}
		return text;
	}

	return painter;
}();


